hotssenseiraw = {
    "name": "SteelSeries Heroes of the Storm (Sensei Raw)",

    "vendor_id": 0x1038,
    "product_id": 0x1390,
    "interface_number": 0,

    "commands": {

        "set_sensitivity1": {
            "description": "Set sensitivity preset 1",
            "cli": ["-s", "--sensitivity1"],
            "command": [0x03, 0x01],
            "value_type": "range",
            "range_min": 90,
            "range_max": 5400,
            "range_increment": 90,
            "value_transform": lambda x: int(x / 90),
            "default": 1530,
        },

        "set_sensitivity2": {
            "description": "Set sensitivity preset 2",
            "cli": ["-S", "--sensitivity2"],
            "command": [0x03, 0x02],
            "value_type": "range",
            "range_min": 90,
            "range_max": 5400,
            "range_increment": 90,
            "value_transform": lambda x: int(x / 90),
            "default": 2520,
        },

        "set_logo_light_effect": {
            "description": "Set the logo light effect",
            "cli": ["-e", "--logo-light-effect"],
            "command": [0x07, 0x01],
            "value_type": "choice",
            "choices": {
                "steady": 0x01,
                "breathslow": 0x02,
                "breathmed": 0x03,
                "breathfast": 0x04,
                "trigger": 0x05,
                1: 0x01,
                2: 0x02,
                3: 0x03,
                4: 0x04,
                5: 0x05,
            },
            "default": "breathmed",
        },

        "set_logo_light_brightness": {
            "description": "Set logo light brightness",
            "cli": ["-E", "--logo-light-brightness"],
            "command": [0x05, 0x01],
            "value_type": "choice",
            "choices": {
                "off": 0x01,
                "low": 0x02,
                "med": 0x03,
                "high": 0x04,
                1: 0x01,
                2: 0x02,
                3: 0x03,
                4: 0x04,
            },
            "default": "high",
        },

        "set_mouse_btn_action": {
            "description": "Set mouse button actions",
            "cli": ["-z", "--set-btns"],
            "command": [0x31, 0x00],
            "value_type": "hotsbtnmap",
        },

        "save": {
            "description": "Save the configuration to the mouse memory",
            "cli": None,
            "command": [0x09, 0x00],
            "value_type": None,
        },

    },

}



from .rival300csgofadeedition import rival300csgofadeedition

rival300csgohyperbeastedition = {
    "name": "SteelSeries Rival 300 CS:GO Hyperbeast Edition",

    "vendor_id": 0x1038,
    "product_id": 0x171a,
    "interface_number": 0,

    # Hyperbeast command mapping is same as CS:GO Fade Edition
    "commands": rival300csgofadeedition["commands"]
}



from .rival300 import rival300

rival300dota2edition = {
    "name": "SteelSeries Rival 300 Dota 2 Edition",

    "vendor_id": 0x1038,
    "product_id": 0x1392,
    "interface_number": 0,

    "commands": rival300["commands"]

}



from io import BytesIO

import hid

from . import debug


HID_REPORT_TYPE_OUTPUT = 0x02
HID_REPORT_TYPE_FEATURE = 0x03


def is_device_plugged(vendor_id, product_id):
    """Returns True if the given HID device is plugged to the computer.

    Arguments:
    vendor_id -- the mouse vendor id (e.g. 0x1038)
    product_id -- the mouse product id (e.g. 0x1710)
    """
    if debug.DEBUG:
        mouse_id = debug.get_debug_profile()
        if mouse_id:
            return (mouse_id.vendor_id == vendor_id
                    and mouse_id.product_id == product_id)
    return len(hid.enumerate(vendor_id, product_id)) > 0


def open_device(vendor_id, product_id, interface_number):
    """Opens and returns the HID device (file-like object). Raise IOError if
    the device or interface is not available.

    Arguments:
    vendor_id -- the mouse vendor id (e.g. 0x1038)
    product_id -- the mouse product id (e.g. 0x1710)
    interface_number -- the interface number (e.g. 0x00)
    """
    # Dry run
    if debug.DEBUG and debug.DRY and is_device_plugged(vendor_id, product_id):
        device = BytesIO()  # Moke the device
        device.send_feature_report = device.write
        return device

    # Real device
    for interface in hid.enumerate(vendor_id, product_id):
        if interface["interface_number"] != interface_number:
            continue
        device = hid.device()
        device.open_path(interface["path"])
        return device

    raise IOError("Unable to find the requested device: %04X:%04X:%02X" % (
        vendor_id, product_id, interface_number))



import pytest

import rivalcfg.command_handlers


class TestTransform(object):

    def test_one_argument(self):
        value = rivalcfg.command_handlers._transform(
                {"value_transform": lambda v: v * 2}, 4)
        assert value == 8

    def test_multiple_arguments(self):
        values = rivalcfg.command_handlers._transform(
                {"value_transform": lambda a, b: (a * 2, b * 4)}, 8, 4)
        assert values == (16, 16)

    def test_one_argument_with_no_transform_function(self):
        value = rivalcfg.command_handlers._transform({}, 4)
        assert value == 4

    def test_multiple_arguments_with_no_transform_function(self):
        values = rivalcfg.command_handlers._transform({}, 8, 4)
        assert values == (8, 4)


class TestChoiceHandler(object):

    @pytest.fixture
    def choice_command(self):
        return {
            "command": [0x01, 0x02],
            "choices": {
                0: 0xAA,
                10: 0xBB,
                20: 0xCC
                }
            }

    @pytest.fixture
    def choice_command_transform(self):
        return {
            "command": [0x02],
            "choices": {1: 0xAB},
            "value_transform": lambda v: v - 1
            }

    def test_valid_choice(self, choice_command):
        bytes_ = rivalcfg.command_handlers.choice_handler(choice_command, 10)
        assert bytes_ == [0x01, 0x02, 0xBB]

    def test_not_valid_choice(self, choice_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.choice_handler(choice_command, 42)

    def test_value_tranform(self, choice_command_transform):
        bytes_ = rivalcfg.command_handlers.choice_handler(
                choice_command_transform, 1)
        assert bytes_ == [0x02, 0xAA]


class TestRgbcolorHandler(object):

    @pytest.fixture
    def rgbcolor_command(self):
        return {"command": [0x01, 0x02]}

    @pytest.fixture
    def rgbcolor_command_transform(self):
        return {
            "command": [0x02],
            "value_transform": lambda r, g, b: (r-1, g+2, b+10)
            }

    def test_valid_color_string(self, rgbcolor_command):
        bytes_ = rivalcfg.command_handlers.rgbcolor_handler(
                rgbcolor_command, "#ff1800")
        assert bytes_ == [0x01, 0x02, 0xFF, 0x18, 0x00]

    def test_not_valid_color_string(self, rgbcolor_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.rgbcolor_handler(
                    rgbcolor_command, "hello")

    def test_valid_color_ints(self, rgbcolor_command):
        bytes_ = rivalcfg.command_handlers.rgbcolor_handler(
                rgbcolor_command, 0xFF, 0x18, 0x00)
        assert bytes_ == [0x01, 0x02, 0xFF, 0x18, 0x00]

    def test_not_valid_color_ints_2_channels(self, rgbcolor_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.rgbcolor_handler(
                    rgbcolor_command, 0xFF, 0x18)

    def test_not_valid_color_ints_wrong_range(self, rgbcolor_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.rgbcolor_handler(
                    rgbcolor_command, -1, 256, 1337)

    def test_not_valid_color_ints_wrong_type(self, rgbcolor_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.rgbcolor_handler(
                    rgbcolor_command, "ff", "18", "00")

    def test_color_command_with_transform(self, rgbcolor_command_transform):
        bytes_ = rivalcfg.command_handlers.rgbcolor_handler(
                rgbcolor_command_transform,  0xFF, 0x18, 0x00)
        assert bytes_ == [0x02, 0xFE, 0x1A, 0x0A]


class TestRgbcolorshiftHandler(object):

    @pytest.fixture
    def rgbcolorshift_command(self):
        return {"command": [0x01, 0x02]}

    @pytest.fixture
    def rgbcolorshift_command_transform(self):
        return {
            "command": [0x02],
            "value_transform": lambda cs, s: (
                (cs[0]+1, cs[1]+1, cs[2]+1, cs[3]+1, cs[4]+1, cs[5]+1), s+1)
            }

    def test_valid_colors(self, rgbcolorshift_command):
        bytes_ = rivalcfg.command_handlers.rgbcolorshift_handler(
                rgbcolorshift_command,
                [[0x01, 0x02, 0x03], [0x06, 0x05, 0x04]],
                0x88)
        assert bytes_ == [
                0x01, 0x02, 0x01, 0x02, 0x03, 0x06, 0x05, 0x04, 0x88, 0x00]
        bytes_ = rivalcfg.command_handlers.rgbcolorshift_handler(
                rgbcolorshift_command, ["010203", "#060504"], 0x88)
        assert bytes_ == [
                0x01, 0x02, 0x01, 0x02, 0x03, 0x06, 0x05, 0x04, 0x88, 0x00]

    def test_valid_colors_and_no_with_tranform(self, rgbcolorshift_command_transform):  # noqa
        bytes_ = rivalcfg.command_handlers.rgbcolorshift_handler(
                rgbcolorshift_command_transform,
                [[0x01, 0x02, 0x03], [0x06, 0x05, 0x04]],
                0x88)
        assert bytes_ == [0x02, 0x02, 0x03, 0x04, 0x07, 0x06, 0x05, 0x89, 0x00]


class TestRangeHandler(object):

    @pytest.fixture
    def range_command(self):
        return {
            "command": [0x01, 0x02],
            "range_min": 50,
            "range_max": 150
            }

    @pytest.fixture
    def range_command_increment(self):
        return {
            "command": [0x02],
            "range_min": 50,
            "range_max": 150,
            "range_increment": 50
            }

    @pytest.fixture
    def range_command_transform(self):
        return {
            "command": [0x03],
            "range_min": 50,
            "range_max": 150,
            "value_transform": lambda v: v * 2
            }

    def test_valid_value(self, range_command):
        bytes_ = rivalcfg.command_handlers.range_handler(range_command, 0x88)
        assert bytes_ == [0x01, 0x02, 0x88]

    def test_value_smaller_than_min(self, range_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.range_handler(range_command, 42)

    def test_value_greater_than_max(self, range_command):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.range_handler(range_command, 151)

    def test_value_multiple_of_increment(self, range_command_increment):
        bytes_ = rivalcfg.command_handlers.range_handler(
                range_command_increment, 0x96)
        assert bytes_ == [0x02, 0x96]

    def test_value_not_multiple_of_increment(self, range_command_increment):
        with pytest.raises(ValueError):
            rivalcfg.command_handlers.range_handler(
                    range_command_increment, 101)

    def test_range_command_with_transform(self, range_command_transform):
        bytes_ = rivalcfg.command_handlers.range_handler(
                range_command_transform, 0x70)
        assert bytes_ == [0x03, 0xE0]


class TestHNoneHandler(object):

    @pytest.fixture
    def none_command(self):
        return {"command": [0x01, 0x02]}

    def test_command_with_no_argument(self, none_command):
        bytes_ = rivalcfg.command_handlers.none_handler(none_command)
        assert bytes_ == [0x01, 0x02]



from .rival import rival

rival300hpomen = {
    "name": "SteelSeries Rival 300 HP Omen Edition",

    "vendor_id": 0x1038,
    "product_id": 0x1718,
    "interface_number": 0,

    "commands": rival["commands"]
}



from .rival300csgofadeedition import rival300csgofadeedition


rival300csgofadeeditionstm32 = {
    "name": "SteelSeries Rival 300 CS:GO Fade Edition (stm32)",

    "vendor_id": 0x1038,
    "product_id": 0x1716,
    "interface_number": 0,

    "commands": rival300csgofadeedition["commands"],

}



from .rival100 import rival100

rival105 = {
    "name": "SteelSeries Rival 105 (Experimental)",

    "vendor_id": 0x1038,
    "product_id": 0x1814,
    "interface_number": 0,

    "commands": rival100["commands"]

}



#!/usr/bin/env python
# encoding: UTF-8

import os
import shutil
import subprocess
from setuptools import setup, find_packages
from setuptools.command.install import install as _install


class install(_install):
    def run(self):
        _install.run(self)
        print("Installing udev rules...")
        if not os.path.isdir("/etc/udev/rules.d"):
            print("WARNING: udev rules have not been installed (/etc/udev/rules.d is not a directory)")
            return
        try:
            shutil.copy("./rivalcfg/data/99-steelseries-rival.rules", "/etc/udev/rules.d/")
        except IOError:
            print("WARNING: udev rules have not been installed (permission denied)")
            return
        try:
            subprocess.call(["udevadm", "trigger"])
        except OSError:
            print("WARNING: unable to update udev rules, please run the 'udevadm trigger' command")
            return
        print("Done!")


long_description = ""

try:
    if os.path.isfile("README.rst"):
        long_description = open("README.rst", "r").read()
    elif os.path.isfile("README.md"):
        long_description = open("README.md", "r").read()
except Exception as error:
    print("Unable to read the README file: " + str(error))


setup(
    name="rivalcfg",
    version="3.12.0",
    description="Configure SteelSeries Rival gaming mice",
    url="https://github.com/flozz/rivalcfg",
    license="WTFPL",

    long_description=long_description,

    author="Fabien LOISON",

    keywords="steelseries rival rival100 mouse",
    platforms=["Linux"],

    packages=find_packages(),

    install_requires=[
        "hidapi>=0.7.99.post20"
    ],

    entry_points={
        "console_scripts": [
            "rivalcfg = rivalcfg.__main__:main"
        ]
    },

    cmdclass={"install": install}
)



senseiraw = {
    "name": "SteelSeries Sensei RAW (Experimental)",

    "vendor_id": 0x1038,
    "product_id": 0x1369,
    "interface_number": 0,

    "commands": {

        "set_logo_light_effect": {
            "description": "Set the logo light effect",
            "cli": ["-e", "--logo-light-effect"],
            "command": [0x07, 0x01],
            "value_type": "choice",
            "choices": {
                "steady": 0x01,
                "breath": 0x03,
                "off": 0x05,
                0: 0x00,
                1: 0x01,
                2: 0x02,
                3: 0x03,
                4: 0x04,
                5: 0x05,
            },
            "default": "steady",
        },

    },

}



from .rival310 import rival310

rival310pubgedition = {
    "name": "SteelSeries Rival 310 PUBG Edition (Experimental)",

    "vendor_id": 0x1038,
    "product_id": 0x1736,
    "interface_number": 0,

    "rgbuniversal_format": rival310["rgbuniversal_format"],

    "commands": rival310["commands"],
}



import pytest

import rivalcfg.helpers


class TestIntToLittleEndianBytearray(object):

    def test_small_number_16bit(self):
        assert rivalcfg.helpers.uint_to_little_endian_bytearray(0x0001, 2) == [0x01, 0x00]  # noqa

    def test_bigger_number_16bit(self):
        assert rivalcfg.helpers.uint_to_little_endian_bytearray(0x0a01, 2) == [0x01, 0x0a]  # noqa

    def test_overflow_number_16bit(self):
        assert rivalcfg.helpers.uint_to_little_endian_bytearray(0xFFFF, 2) == [0xFF, 0xFF]  # noqa
        with pytest.raises(ValueError):
            rivalcfg.helpers.uint_to_little_endian_bytearray(0xABCDEF, 2)
